from zope.interface import implements
from zgeo.atom.interfaces import ILink


class Link(object):
    implements(ILink)

    def __init__(self, href, rel=None, type='application/atom+xml'):
        self.href = href
        self.rel = rel
        self.type = type
    



from zope import component
from zope.interface import implements, Interface
from zope.app.publication.http import HTTPPublication
from zope.app.publication.requestpublicationfactories import HTTPFactory
from zope.publisher.http import HTTPRequest
from zope.publisher.interfaces import NotFound
from zope.publisher.interfaces.http import IHTTPCredentials
from zope.publisher.interfaces.http import IHTTPRequest
from zope.publisher.interfaces.http import IHTTPApplicationRequest
from zope.publisher.interfaces.http import IHTTPPublisher
from zope.app.container.interfaces import IReadContainer
from zope.filerepresentation.interfaces import IFileFactory, IWriteFile
from zope.filerepresentation.interfaces import IWriteDirectory
from zope.traversing.api import getParent
from zgeo.atom.browser import absoluteURL, LinkEntry
from zgeo.atom.interfaces import IAtomEntryNameFactory


class IAtomPubRequestFactory(Interface):
    pass


class IAtomPubRequest(IHTTPRequest):
    pass


class AtomPubRequest(HTTPRequest):
    implements(IHTTPCredentials, IAtomPubRequest, IHTTPApplicationRequest)


class AtomPubPublication(HTTPPublication):
    pass


class AtomPubPublicationFactory(HTTPFactory):
    def __call__(self):
        request_class = component.queryUtility(
            IAtomPubRequestFactory, default=AtomPubRequest
            )
        return request_class, AtomPubPublication


class AtomPubTraverser(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def publishTraverse(self, request, name):
        if name in ['atompub-collection', 'atom-entry']:
            return self.context
        elif IReadContainer.providedBy(self.context):
            item = self.context.get(name)
            if item is not None:
                return item

        # fall back to views
        view = component.queryMultiAdapter((self.context, request), name=name)
        if view is not None:
            return view

        # give up and return a 404 Not Found error page
        raise NotFound(self.context, name, request)


class AtomCollectionPOST(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def POST(self):
        """Add a new item to the container."""
        name = component.queryMultiAdapter(
                    (self.context, self.request),
                    IAtomEntryNameFactory,
                    ).chooseName()
        body = self.request.bodyStream
        factory = component.getAdapter(
            self.context, IFileFactory, 
            name='application/atom+xml;type=entry'
            )
        placemark = factory(name, 'application/atom+xml;type=entry', body)
        self.context[name] = placemark

        response = self.request.response
        response.setStatus(201)
        loc = "%s/atom-entry" % absoluteURL(self.context[name], self.request)
        response.setHeader('Location', loc)
        return LinkEntry(placemark, self.request)()


class AtomMemberPUT(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def PUT(self):
        body = self.request.bodyStream
        adapter = component.getAdapter(
            self.context, IWriteFile, name='application/atom+xml;type=entry'
            )
        adapter.write(body.read())
        self.request.response.setStatus(200)
        return ''


class AtomMemberDELETE(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def DELETE(self):
        parent = getParent(self.context)
        adapter = component.queryAdapter(
            parent, IWriteDirectory,
            name='application/atom+xml;type=entry',
            default=parent
            )
        adapter.__delitem__(self.context.__name__)
        self.request.response.setStatus(200)
        return ''



from zope.interface import Interface, Attribute


class IAtomMetadata(Interface):
    
    """See http://tools.ietf.org/html/rfc4287, section 4.2.6.
    """

    id = Attribute("""atom:id""")
    categories = Attribute("""List of categories""")


class IWritableAtomMetadata(Interface):

    def setId():
        """Set and return a urn:uuid id."""


class IWriteAtomMetadata(IAtomMetadata, IWritableAtomMetadata):
    """Write atomid as well as read."""


class IAtomEntryNameFactory(Interface):

    def chooseName():
        """Returns the new name within the collection."""


class ICategory(Interface):

    """See http://tools.ietf.org/html/rfc4287, section 4.2.2.
    """

    term = Attribute("""See 4.2.2.1.""")
    scheme = Attribute("""See 4.2.2.2.""")
    label = Attribute("""See 4.2.2.3.""")


class ILink(Interface):
    
    """See http://tools.ietf.org/html/rfc4287.
    """

    href = Attribute("""A URI""")
    type = Attribute("""Content type of the linked resource""")
    rel = Attribute("""Link relationship: 'alternate', 'self', etc.""")

    
class IAtomBase(Interface):
    
    """See http://tools.ietf.org/html/rfc4287.
    """

    author = Attribute("""A mapping with name, URI, and email keys""")
    id = Attribute("""A universally unique identifier""")
    title = Attribute("""A human readable text""")
    updated = Attribute("""A RFC 3339 date/time string""")
    links = Attribute("""A dict of links""")


class IEntry(IAtomBase):

    """See http://tools.ietf.org/html/rfc4287.
    """

    summary = Attribute("""A human readable text summary""")
    updated = Attribute("""A RFC 3339 date/time string""")

    # geographic elements
    coords_georss = Attribute("""GML coordinate encoding of the location""")
    hasLineString = Attribute("""Boolean, True if has a line location""")
    hasPoint = Attribute("""Boolean, True if has a point location""")
    hasPolygon = Attribute("""Boolean, True if has a polygon location""")


class IFeed(IAtomBase):

    """See http://tools.ietf.org/html/rfc4287.
    """

    entries = Attribute("""An iterator over feed entries""")


class ISubscriptionFeed(IFeed):

    """See http://tools.ietf.org/html/rfc5005.
    
    Must contain a 'previous-archive' link.
    """


class IArchiveFeed(IFeed):

    """See http://tools.ietf.org/html/rfc5005.
    
    Must contain 'current', 'next-archive', and 'previous-archive' links.
    """


class IPagingFeed(IFeed):

    """See http://tools.ietf.org/html/rfc5005.
    
    Must contain 'first', 'last', 'previous', and 'next' links.
    """


class ISearchFeed(IPagingFeed):

    bounds = Attribute("""A (minx, miny, maxx, maxy) tuple""")
    page = Attribute("""Integer page number""")


class IAtomPubPOSTable(Interface):
    """Marker for collections."""

class IAtomPubPUTable(Interface):
    """Marker for editable members."""

class IAtomPubDELETEable(Interface):
    """Marker for deleteable members."""

class IAtomPublishable(IAtomPubPUTable, IAtomPubDELETEable):
    """Marker."""




from zope.component import adapts
from zope.interface import implements
from zgeo.atom.interfaces import IAtomMetadata, IWriteAtomMetadata
from zope.annotation.interfaces import IAnnotations, IAnnotatable
from persistent.dict import PersistentDict
import uuid


KEY = 'zgeo.atom.metadata'


class AtomAnnotatableAdapter(object):
    implements(IWriteAtomMetadata)
    adapts(IAnnotatable)

    def __init__(self, context):
        self.context = context
        annotations = IAnnotations(context)
        self.atom = annotations.get(KEY, None)
        if not self.atom:
            annotations[KEY] = PersistentDict()
            self.atom = annotations[KEY]
            self.atom['id'] = None

    def _get_id(self):
        return self.atom['id']

    def _set_id(self, value):
        self.atom['id'] = value

    def setId(self):
        """Set and return a urn:uuid."""
        self.atom['id'] = 'urn:uuid:%s' % str(uuid.uuid4())
        return self.atom['id']

    id = property(_get_id, _set_id)



import unittest
from zope.testing import doctest
from zope.component import provideAdapter
from zope.app.testing.setup import placefulSetUp, placefulTearDown
from zope.app.testing.functional import FunctionalDocFileSuite
from zope.interface import implements, Interface, alsoProvides
from zope.app.intid.interfaces import IIntIds
from zope.app.catalog.interfaces import ICatalog
from zgeo.spatialindex.bounds import Bounded
from zgeo.spatialindex.catalog import Catalog
from zgeo.spatialindex.interfaces import IBounded, ISpatiallyCataloged
from zgeo.atom.example import PlacesFolder, Placemark
from zgeo.atom.testing import AtomLayer
from zgeo.atom.metadata import AtomAnnotatableAdapter
from zope.annotation.attribute import AttributeAnnotations
from zope.annotation.interfaces import IAttributeAnnotatable


optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS | doctest.REPORT_ONLY_FIRST_FAILURE

def setUp(test):
    placefulSetUp()
    provideAdapter(Bounded)
    provideAdapter(AttributeAnnotations)
    provideAdapter(AtomAnnotatableAdapter)

    from zope.app.folder import Folder, rootFolder
    root = rootFolder()
    test.globs['root'] = root
    places = PlacesFolder()
    root['places'] = places
    places.__name__ = 'places'
    test.globs['places'] = places

    import os
    os.environ['CLIENT_HOME'] = os.getcwd()

    catalog = Catalog()
    from zope.app.testing import ztapi
    ztapi.provideUtility(ICatalog, catalog)
    from zgeo.spatialindex.index import BoundsIndex
    catalog['bounds'] = BoundsIndex('bounds', IBounded)
    test.globs['catalog'] = catalog

    placemark = Placemark('a', "Place marked 'A'", -105.08, 40.59)
    places['a'] = placemark
    placemark.__parent__ = places
    placemark.__name__ = 'a'
    test.globs['placemark'] = placemark

    catalog.index_doc(1, placemark)
    
    class Ids:
        implements(IIntIds)
        def __init__(self, data):
            self.data = data
        def getObject(self, id):
            return self.data[int(id)]
        def __iter__(self):
            return self.data.iterkeys()
    intids = Ids({1: placemark})
    ztapi.provideUtility(IIntIds, intids)

    alsoProvides(places, ISpatiallyCataloged)
    alsoProvides(places, IAttributeAnnotatable)
    alsoProvides(placemark, IAttributeAnnotatable)

def tearDown(test):
    placefulTearDown()

def test_suite():
    atomdocs = FunctionalDocFileSuite(
            'atom-docs.txt',
            optionflags=optionflags,
            )
    atomdocs.layer = AtomLayer
    atompub = FunctionalDocFileSuite(
            'atom-pub.txt',
            optionflags=optionflags,
            )
    atompub.layer = AtomLayer

    return unittest.TestSuite([
        doctest.DocFileSuite(
            'README.txt',
            package='zgeo.atom',
            setUp=setUp,
            tearDown=tearDown,
            optionflags=optionflags,
            ),
        atomdocs,
        atompub,
        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from zope.component import adapts
from zope.interface import implements, alsoProvides
from zope.dublincore.interfaces import ICMFDublinCore
from zope.dublincore.interfaces import IWriteZopeDublinCore, IZopeDublinCore
from zgeo.geographer.interfaces import IGeoreferenceable, IGeoreferenced
from zope.filerepresentation.interfaces import IFileFactory, IWriteFile
from zope.app.folder import Folder
from zope.app.folder.interfaces import IFolder
from zgeo.atom import feedparser
from zgeo.atom.interfaces import IAtomPublishable
from zgeo.atom.publication import IAtomPubRequest
from zgeo.atom.interfaces import IAtomEntryNameFactory
from zgeo.geographer.interfaces import IWriteGeoreferenced
import datetime
import re
import uuid

class Placemark(object):
    implements((IGeoreferenceable, IGeoreferenced, ICMFDublinCore))
    
    def __init__(self, id, summary, long, lat):
        self.id = id
        self.summary = summary
        self.type = 'Point'
        self.coordinates = (long, lat)
    
    def Title(self):
        return self.__name__.capitalize()
    
    def Description(self):
        return self.summary
    
    def CreationDate(self):
        return '2007-12-07 12:00:00'
    
    def ModificationDate(self):
        return '2007-12-07 12:01:00'

class PlacesFolder(Folder):
    implements(ICMFDublinCore)
    
    def Title(self):
        return self.__name__.capitalize()
    
    def Description(self):
        return 'Test Places'
    
    def CreationDate(self):
        return '2007-12-07 12:00:00'
    
    def ModificationDate(self):
        return '2007-12-07 12:01:00'

class AbsoluteURL(object):
    
    def __init__(self, context, request):
        self.context = context
    
    def __call__(self):
        return self.context.absolute_url()

class AtomFileFactory(object):
    """Adapts a generic content object"""
    implements(IFileFactory)
    adapts(IFolder)

    def __init__(self, context):
        self.context = context
    
    def __call__(self, name, content_type, data):
        feed = feedparser.parse(data)
        entry = feed.entries[0]
        title = entry.get('title', u'Untitled')
        summary = entry.get('summary', u'Unsummarized')
        where = entry.where
        p = Folder()
        dc = IWriteZopeDublinCore(p)
        dc.title = title
        dc.description = summary
        dc.creator = u'Grok'
        geo = IWriteGeoreferenced(p)
        geo.setGeoInterface(where['type'], where['coordinates'])
        now = datetime.datetime.now()
        dc.created = now
        p.__name__ = name
        alsoProvides(p, IAtomPublishable)
        return p

class AtomWriteFile(object):    
    implements(IWriteFile)
    adapts(IFolder)
    
    def __init__(self, context):
        self.context = context
    
    def write(self, data):
        dc = IWriteZopeDublinCore(self.context)
        geo = IWriteGeoreferenced(self.context)
        feed = feedparser.parse(data)
        entry = feed.entries[0]
        title = entry.get('title')
        if title is not None:
            dc.title = title
        summary = entry.get('summary')
        if summary is not None:
            dc.description = summary
        where = entry.get('where')
        if where is not None:
            geo.setGeoInterface(where['type'], where['coordinates'])

class AtomEntryNamer(object):
    implements(IAtomEntryNameFactory)
    adapts(IFolder, IAtomPubRequest)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_unique_name(self, candidate):
        # Make a unique name from candidate by suffixing
        if candidate not in self.context:
            return candidate
        m = re.search(r"(.+)\.(\d+)$", candidate)
        if not m:
            x = "%s.1" % candidate
        else:
            x = "%s.%d" % (m.groups[0], int(m.groups[1])+1)
        return self.get_unique_name(x)

    def chooseName(self):
        slug = self.request.getHeader('Slug')
        if slug is not None:
            name = slug.strip().lower()
            name = self.get_unique_name(re.sub('\W+', '-', name))
        else:
            name = str(uuid.uuid1())
        return name



from setuptools import setup, find_packages

version = '0.4.1'
readme_text = open('README.txt', 'rb').read()

setup(name='zgeo.atom',
      version=version,
      description="Atom syndication and AtomPub with GeoRSS",
      long_description=readme_text,
      classifiers=[
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Scientific/Engineering :: GIS",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License"
        ],
      keywords='gis geography geospatial georss atom atompub',
      author='Sean Gillies',
      author_email='sgillies@frii.com',
      url='http://trac.gispython.org/projects/PrimaGIS/wiki/zgeo.atom',
      license='BSD',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['zgeo'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'setuptools',
        'zgeo.geographer>=0.3',
      ],
      extras_require = {
        'bbox search': ['zgeo.spatialindex>=0.3'],
        },
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from zope.traversing.browser.interfaces import IAbsoluteURL
from zope.dublincore.interfaces import ICMFDublinCore

try:
    from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
    raise Exception, "Five's ViewPageTemplateFile doesn't work with named templating"
except:
    from zope.app.pagetemplate import ViewPageTemplateFile

from zope.interface import implements
from zope.publisher.browser import BrowserPage
from zope.formlib.namedtemplate import NamedTemplate
from zope.formlib.namedtemplate import NamedTemplateImplementation
from zope.component import getMultiAdapter, queryAdapter
import zope.security.proxy

from zgeo.geographer.interfaces import IGeoreferenced
from zgeo.atom.interfaces import IAtomBase, IEntry, ILink
from zgeo.atom.interfaces import IFeed, ISubscriptionFeed, ISearchFeed
from zgeo.atom.interfaces import IWriteAtomMetadata, IAtomPublishable
from zgeo.atom.interfaces import IAtomPubPOSTable
from zgeo.atom.link import Link
import zope.datetime

# zgeo.spatialindex is required to use a bbox parameter with paging search
# feeds
try:
    from zgeo.spatialindex.site import get_catalog
except ImportError:
    def noner(arg):
        return None
    get_catalog = noner

def coords_to_georss(geom):
    gtype = geom.type
    if gtype == 'Point':
        coords = (geom.coordinates,)
    elif gtype == 'Polygon':
        coords = geom.coordinates[0]
    else:
        coords = geom.coordinates
    tuples = ('%f %f' % (c[1], c[0]) for c in coords)
    return ' '.join(tuples)

def rfc3339(date):
    ts = zope.datetime.time(date)
    return zope.datetime.iso8601_date(ts)
    # Convert ISO or RFC 3339 datetime strings to RFC 3339
    # Zope's ZDCAnnotatableAdapter gives RFC 3339. Lose the seconds precision
    #if str(date).find('T') == 10:
    #    s = date.split('.')[0]
    # Plone's AT content types give ISO
    #else:
    #    t = time.strptime(date, '%Y-%m-%d %H:%M:%S')
    #    s =  time.strftime('%Y-%m-%dT%H:%M:%S', t)
    #tz = '%03d:00' % -int(time.timezone/3600)
    #return s + tz

def absoluteURL(ob, request):
    return getMultiAdapter((ob, request), IAbsoluteURL)()


class NullGeometry(object):
    type = None
    coordinates = None


class NullGeoItem(object):
    id = None
    properties = None

    def __init__(self):
        self.geometry = NullGeometry()


class AtomBase(BrowserPage):

    """Not to be instantiated.
    """
    implements(IAtomBase)

    @property
    def id(self):
        context = zope.security.proxy.removeSecurityProxy(self.context)
        atom = IWriteAtomMetadata(context)
        if atom.id is None:
            return atom.setId()
        return atom.id

    @property
    def title(self):
        return self.dc.Title()

    @property
    def updated(self):
        return rfc3339(self.dc.ModificationDate())


    @property
    def author(self):
        return {
            'name': self.dc.Creator()
,
            'uri': '',
            'email': ''
            }

    @property
    def links(self):
        """Override this."""
        raise NotImplementedError


class LinkEntry(AtomBase):

    implements(IEntry)

    __name__ = 'atom-entry'
    template = NamedTemplate('template-atom-entry')

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.dc = zope.security.proxy.removeSecurityProxy(
            ICMFDublinCore(self.context)
            )
        try:
            self.geom = IGeoreferenced(self.context)
        except:
            self.geom = NullGeometry()

    @property
    def published(self):
        return rfc3339(self.dc.CreationDate())

    @property
    def summary(self):
        return self.dc.Description()

    @property
    def links(self):
        items = {
            'alternate': Link(
                absoluteURL(self.context, self.request),
                rel='alternate',
                type='text/html')
            }
        if IAtomPublishable.providedBy(self.context):
            items['edit'] = Link(
                "%s/atom-entry" % absoluteURL(self.context, self.request),
                rel='edit',
                type='application/atom+xml;type=entry')
        return items

    @property
    def hasPoint(self):
        return int(self.geom.type == 'Point')

    @property
    def hasLineString(self):
        return int(self.geom.type == 'LineString')

    @property
    def hasPolygon(self):
        return int(self.geom.type == 'Polygon')

    @property
    def coords_georss(self):
        return coords_to_georss(self.geom)

    def __call__(self):
        return self.template().encode('utf-8')


class FeedBase(AtomBase):

    implements(IFeed)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.dc = zope.security.proxy.removeSecurityProxy(
            ICMFDublinCore(self.context)
            )

    @property
    def links(self):
        raise NotImplementedError

    @property
    def entries(self):
        context = zope.security.proxy.removeSecurityProxy(self.context)
        for item in context.values():
            yield LinkEntry(item, self.request)

    def collection_href(self):
        if IAtomPubPOSTable.providedBy(self.context):
            return '%s/atompub-collection' % absoluteURL(
                                                self.context, self.request)
        return None


class SubscriptionFeed(FeedBase):

    implements(ISubscriptionFeed)

    __name__ = 'atom-subscription-feed'
    template = NamedTemplate('template-atom-subscription-feed')

    @property
    def links(self):
        return {
            'alternate': Link(
                            absoluteURL(self.context, self.request),
                            rel='alternate',
                            type='text/html'
                            ),
            'self': Link(
                self.request.getURL(),
                rel='self',
                type='application/atom+xml'
                ),
            'previous-archive': Link('None', rel='previous-archive')
            }

    def __call__(self):
        return self.template().encode('utf-8')


class SearchFeed(FeedBase):

    implements(ISearchFeed)

    __name__ = 'atom-search-feed'
    template = NamedTemplate('template-atom-search-feed')
    page_size = 20

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.dc = ICMFDublinCore(self.context)
        self.catalog = None
        self.bounds = None
        self.page = 1
        self.num_results = 0
        self.num_pages = 0
        self.results = []

    def parse_bbox(self, bbox=None):
        if bbox is None:
            b = self.request.form.get('bbox')
            if b is None:
                return None
        else:
            b = bbox
        return tuple(float(x) for x in b.split(','))

    def _first_link(self):
        url = "%s/@@%s" % (
            absoluteURL(self.context, self.request), self.__name__
            )
        if self.bounds:
            url = "%s?bbox=%f,%f,%f,%f" % ((url,) + self.bounds)
        return url

    def _last_link(self):
        url = "%s/@@%s?page=-1" % (
            absoluteURL(self.context, self.request), self.__name__
            )
        if self.bounds:
            url = "%s&bbox=%f,%f,%f,%f" % ((url,) + self.bounds)
        return url

    def _previous_link(self):
        if self.page == 1:
            return 'None'
        url = "%s/@@%s?page=%d" % (
            absoluteURL(self.context, self.request),
            self.__name__,
            self.page - 1
            )
        if self.bounds:
            url = "%s&bbox=%f,%f,%f,%f" % ((url,) + self.bounds)
        return url

    def _next_link(self):
        if self.page == -1 or self.page >= self.num_pages:
            return 'None'
        url = "%s/@@%s?page=%d" % (
            absoluteURL(self.context, self.request),
            self.__name__,
            self.page + 1
            )
        if self.bounds:
            url = "%s&bbox=%f,%f,%f,%f" % ((url,) + self.bounds)
        return url

    def update(self):
        self.bounds = self.parse_bbox()
        if self.bounds is not None:
            self.catalog = get_catalog(self.context)
            if self.catalog is None:
                raise Exception, "Spatial search is not supported in the absence of a spatial catalog"
            results = self.catalog.searchResults(bounds=self.bounds)
            num_results = len(results)
            self.results = results
        else:
            results = list(self.context.values())
            num_results = len(results)
            self.results = results
        
        self.num_pages = num_results/self.page_size + num_results%self.page_size
        self.num_results = num_results
        
        page = int(self.request.form.get('page', 1))
        if page > 1 and page > self.num_pages:
            raise Exception, "Page number exceeds number of pages"
        elif page < 0 and -page > self.num_pages:
            raise Exception, "Page number exceeds number of pages"
        else:
            self.page = page

    @property
    def entries(self):
        if self.page >= 0:
            begin = (self.page-1) * self.page_size
            end = begin + self.page_size
        else:
            begin = self.num_results + (self.page * self.page_size)
            end = begin + self.page_size
        if end > self.num_results: end = self.num_results
        for result in list(self.results)[begin:end]:
            yield LinkEntry(result, self.request)

    @property
    def links(self):
        return {
            'alternate': Link(
                absoluteURL(self.context, self.request),
                rel='alternate',
                type='text/html'
                ),
            'self': Link(
                '%s/@@%s' % (absoluteURL(self.context, self.request), self.__name__),
                rel='self',
                type='application/atom+xml'
                ),
            'first': Link(self._first_link(), rel='first'),
            'last': Link(self._last_link(), rel='last'),
            'previous': Link(self._previous_link(), rel='previous'),
            'next': Link(self._next_link(), rel='next'),
            }

    def __call__(self):
        self.update()
        return self.template().encode('utf-8')


# Named template implementations

entry_template = NamedTemplateImplementation(
    ViewPageTemplateFile('entry.pt')
    )

subscription_feed_template = NamedTemplateImplementation(
    ViewPageTemplateFile('subscription_feed.pt')
    )

search_feed_template = NamedTemplateImplementation(
    ViewPageTemplateFile('search_feed.pt')
    )

